
#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4PhysListFactory.hh"
#include "G4VModularPhysicsList.hh"

#include "G4VisExecutive.hh"
#include "G4TrajectoryDrawByParticleID.hh"

#include "G4UIExecutive.hh"

#include "DetectorConstruction.hh"

#include "PrimaryGeneratorActionBasic.hh"
#include "RunActionBasic.hh"
#include "EventActionBasic.hh"

#include "TAnalysisInterface.hh"
#include "TAnalysisBulk.hh"

#include "QGSP_BERT.hh"
#include "QGSP_BERT_HP.hh"
#include "QGSP_BERT_XSHP.hh"
#include "QGSP_INCLXX.hh"
#include "QGSP_MENATE_R.hh"
#include "LHEP.hh"
#include "LHEP_EMV.hh"

#include "G4EmStandardPhysics.hh"
#include "G4EmStandardPhysics_option1.hh"
#include "G4EmStandardPhysics_option2.hh"
#include "G4EmStandardPhysics_option3.hh"
#include "G4EmLivermorePhysics.hh"

// argv[1]: macro name
int main(int argc,char** argv)
{
  // Construct the default run manager
  G4RunManager* runManager = new G4RunManager;

  // set detector construction class
  runManager->SetUserInitialization(new DetectorConstruction);

  // choose physics model
  //  G4VModularPhysicsList* modularPhysicsList = new QGSP_BERT;
  G4VModularPhysicsList* modularPhysicsList = new QGSP_BERT_XSHP;
  //  G4VModularPhysicsList* modularPhysicsList = new QGSP_MENATE_R;
  //  G4VModularPhysicsList* modularPhysicsList = new QGSP_BERT_HP;
  //  G4VModularPhysicsList* modularPhysicsList = new QGSP_INCLXX;
  //  G4VModularPhysicsList* modularPhysicsList = new LHEP;
  //  G4VModularPhysicsList* modularPhysicsList = new LHEP_EMV;

  // change EM physics
  //  modularPhysicsList->ReplacePhysics(new G4EmStandardPhysics_option3);
  //  modularPhysicsList->ReplacePhysics(new G4EmLivermorePhysics);

  // set physics construction class
  runManager->SetUserInitialization(modularPhysicsList);

  runManager->Initialize();

  // set user action classes
  runManager->SetUserAction(new PrimaryGeneratorActionBasic);
  TAnalysisInterface* analysisInterface = new TAnalysisBulk;
  runManager->SetUserAction(new RunActionBasic(analysisInterface));
  runManager->SetUserAction(new EventActionBasic(analysisInterface));

  G4VisManager* visManager = 0;
  visManager = new G4VisExecutive;
  visManager->Initialize();

  G4TrajectoryDrawByParticleID* trajectoryModel = new G4TrajectoryDrawByParticleID;
  trajectoryModel->Set("proton","red");
  trajectoryModel->Set("antiproton","yellow");
  trajectoryModel->Set("neutron","green");
  trajectoryModel->Set("e-","blue");
  trajectoryModel->Set("e+","cyan");
  trajectoryModel->Set("gamma","white");
  trajectoryModel->Set("mu-","magenta");      
  trajectoryModel->Set("mu+","magenta");      
  trajectoryModel->Set("pi-","black");      
  trajectoryModel->Set("pi+","black");      

  visManager->RegisterModel(trajectoryModel);

  if(argc==1){  // interactive mode
    G4UIExecutive* uiExecutive = new G4UIExecutive(argc, argv);
    uiExecutive->SessionStart();
    delete uiExecutive;
  }else{       // Batch mode
    G4String command = "/control/execute ";
    G4String fileName = argv[1];
    G4UImanager* UImanager = G4UImanager::GetUIpointer();
    UImanager->ApplyCommand(command+fileName);
  }

  delete visManager;
  delete runManager;

  return 0;
}
