#ifndef NEUTRONDETECTORCONSTRUCTION_HH
#define NEUTRONDETECTORCONSTRUCTION_HH

#include "G4VUserDetectorConstruction.hh"
#include "G4ThreeVector.hh"

#include "TVector3.h"

class G4LogicalVolume;
class G4VSensitiveDetector;
class G4Material;

class DetectorConstruction;
class DetectorConstructionNEBULA;
class NeutronDetectorConstructionMessenger;

class NeutronDetectorConstruction : public G4VUserDetectorConstruction
{
  friend class DetectorConstruction;
  
public:
  NeutronDetectorConstruction();
  virtual ~NeutronDetectorConstruction();

  G4VPhysicalVolume* Construct();
  virtual G4LogicalVolume* ConstructSub();

  void CopyG4ThreeVectorToTVector3(const G4ThreeVector& threeVector, TVector3& vector3);

  void UpdateGeometry();

  void SetNeutNumber(G4int val);
  void SetNeutSizeX(G4double val);
  void SetNeutSizeY(G4double val);
  void SetNeutSizeZ(G4double val);
  void SetNeutGap(G4double val);
  void SetNeutRotationMode(G4int val);

  void SetVetoNumber(G4int val);
  void SetVetoSizeX(G4double val);
  void SetVetoSizeY(G4double val);
  void SetVetoSizeZ(G4double val);
  void SetVetoGapY(G4double val);
  void SetVetoGapZ(G4double val);
  void SetVetoRotationMode(G4int val);

  void SetNeutLayerNumber(G4int val);
  void SetNeutLayerGap(G4double val);

  void SetNeutVetoGap(G4double val);

  void SetStuckNumber(G4int val);
  void SetStuckGap(G4double val);
  void SetStuckInterval(G4double val);

  void SetPositionX(G4double val);
  void SetPositionY(G4double val);
  void SetPositionZ(G4double val);

  G4int GetNeutNumber(){return fNeutNumber;}
  G4ThreeVector GetNeutSize(){return fNeutSize;}
  G4double GetNeutGap(){return fNeutGap;}
  G4int GetNeutRotationMode(){return fNeutRotationMode;}

  G4int GetVetoNumber(){return fVetoNumber;}
  G4ThreeVector GetVetoSize(){return fVetoSize;}
  G4ThreeVector GetVetoGap(){return fVetoGap;}
  G4int GetVetoRotationMode(){return fVetoRotationMode;}

  G4int GetNeutLayerNumber(){return fNeutLayerNumber;}
  G4double GetNeutLayerGap(){return fNeutLayerGap;}

  // veto layer number is always 1
  G4double GetNeutVetoGap(){return fNeutVetoGap;}

  G4int GetStuckNumber(){return fStuckNumber;}
  G4double GetStuckGap(){return fStuckGap;}

  G4ThreeVector GetStuckSize(){return fStuckSize;}
  G4ThreeVector GetNeutronDetectorSize(){return fNeutronDetectorSize;}

  G4ThreeVector GetPosition(){return fPosition;}

  G4double GetNeutFaceZ(){return fNeutFaceZ;}

protected:
  NeutronDetectorConstructionMessenger* fNeutronDetectorConstructionMessenger;

  G4int fNeutNumber;
  G4ThreeVector fNeutSize;
  G4double fNeutGap;
  G4int fNeutRotationMode;

  G4int fVetoNumber;
  G4ThreeVector fVetoSize;
  G4ThreeVector fVetoGap;
  G4int fVetoRotationMode;

  G4int fNeutLayerNumber;
  G4double fNeutLayerGap;

  // veto layer number is always 1
  G4double fNeutVetoGap;

  G4int fStuckNumber;
  G4double fStuckGap;

  G4ThreeVector fStuckSize;
  G4ThreeVector fNeutronDetectorSize;

  G4ThreeVector fPosition;

  G4double fNeutFaceZ;

  G4LogicalVolume* fLogicNeut;
  G4LogicalVolume* fLogicNeutLayer;
  G4LogicalVolume* fLogicStuck;
  G4LogicalVolume* fLogicVeto;
  G4LogicalVolume* fLogicVetoLayer;
  G4LogicalVolume* fLogicNeutronDetector;
  G4LogicalVolume* fLogicWorld;

  G4VSensitiveDetector* fScintillatorSD;

  G4Material* fNeutMaterial;
  G4Material* fNeutLayerMaterial;
  G4Material* fStuckMaterial;
  G4Material* fVetoMaterial;
  G4Material* fVetoLayerMaterial;
  G4Material* fNeutronDetectorMaterial;
  G4Material* fWorldMaterial;
};

#endif

