#include "ActionBasicMessenger.hh"
#include "PrimaryGeneratorActionBasic.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithABool.hh"
#include "G4UIcmdWithAString.hh"
#include "G4UIcmdWithADoubleAndUnit.hh"

ActionBasicMessenger* ActionBasicMessenger::fActionBasicMessenger(0);

ActionBasicMessenger* ActionBasicMessenger::GetInstance()
{
  if(fActionBasicMessenger == 0){
    fActionBasicMessenger = new ActionBasicMessenger;
  }
  return fActionBasicMessenger;
}

ActionBasicMessenger::ActionBasicMessenger()
  : fRunName("run"), fUseTree(false), fTreeName("tree"), fTreeTitle("simulation tree"),
    fPrimaryGeneratorActionBasic(0)
{
  fActionDir = new G4UIdirectory("/action/");
  fActionDir->SetGuidance(" Set Action Parameters.");

  fRunNameCmd = new G4UIcmdWithAString("/action/RunName",this);
  fRunNameCmd->SetGuidance("Set run name");
  fRunNameCmd->SetParameterName("runName",false);
  fRunNameCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fUseTreeCmd = new G4UIcmdWithABool("/action/UseTree",this);
  fUseTreeCmd->SetGuidance("Set use Tree or not");
  fUseTreeCmd->SetParameterName("useTree",false);
  fUseTreeCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fTreeNameCmd = new G4UIcmdWithAString("/action/TreeName",this);
  fTreeNameCmd->SetGuidance("Set tree name");
  fTreeNameCmd->SetParameterName("treeName",false);
  fTreeNameCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fTreeTitleCmd = new G4UIcmdWithAString("/action/TreeTitle",this);
  fTreeTitleCmd->SetGuidance("Set tree title");
  fTreeTitleCmd->SetParameterName("treeTitle",false);
  fTreeTitleCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fBeamTypeCmd = new G4UIcmdWithAString("/action/BeamType",this);
  fBeamTypeCmd->SetGuidance("Set beam type");
  fBeamTypeCmd->SetParameterName("beamType",false);
  fBeamTypeCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fBeamEnergyCmd = new G4UIcmdWithADoubleAndUnit("/action/BeamEnergy",this);
  fBeamEnergyCmd->SetGuidance("Set beam energy");
  fBeamEnergyCmd->SetParameterName("beamEnergy",false);
  fBeamEnergyCmd->SetUnitCategory("Energy");
  fBeamEnergyCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fBeamSigmaCmd = new G4UIcmdWithADoubleAndUnit("/action/BeamSigma",this);
  fBeamSigmaCmd->SetGuidance("Set beam sigma");
  fBeamSigmaCmd->SetParameterName("beamSigma",false);
  fBeamSigmaCmd->SetUnitCategory("Energy");
  fBeamSigmaCmd->AvailableForStates(G4State_PreInit,G4State_Idle);

  fBeamThetaCmd = new G4UIcmdWithADoubleAndUnit("/action/BeamTheta",this);
  fBeamThetaCmd->SetGuidance("Set beam theta");
  fBeamThetaCmd->SetParameterName("beamTheta",false);
  fBeamThetaCmd->SetUnitCategory("Angle");
  fBeamThetaCmd->AvailableForStates(G4State_PreInit,G4State_Idle);
}

ActionBasicMessenger::~ActionBasicMessenger()
{
  delete fRunNameCmd;
  delete fUseTreeCmd;
  delete fTreeNameCmd;
  delete fTreeTitleCmd;
  delete fBeamTypeCmd;
  delete fBeamEnergyCmd;
  delete fBeamSigmaCmd;
  delete fBeamThetaCmd;
  delete fActionDir;

  fActionBasicMessenger = 0;
}

void ActionBasicMessenger::Register(PrimaryGeneratorActionBasic* primaryGeneratorActionBasic)
{
  fPrimaryGeneratorActionBasic = primaryGeneratorActionBasic;
}

void ActionBasicMessenger::SetNewValue(G4UIcommand* command, G4String newValue)
{
  if(command == fRunNameCmd){
    fRunName = newValue;
  }else if(command == fUseTreeCmd){
    fUseTree = fUseTreeCmd->GetNewBoolValue(newValue);
  }else if(command == fTreeNameCmd){
    fTreeName = newValue;
  }else if(command == fTreeTitleCmd){
    fTreeTitle = newValue;
  }else if(command == fBeamTypeCmd){
    if(!fPrimaryGeneratorActionBasic->SetBeamType(newValue)) G4Exception(0, 0, FatalException, 0);
  }else if(command == fBeamEnergyCmd){
    if(!fPrimaryGeneratorActionBasic->SetBeamEnergy(fBeamEnergyCmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fBeamSigmaCmd){
    if(!fPrimaryGeneratorActionBasic->SetBeamSigma(fBeamSigmaCmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }else if(command == fBeamThetaCmd){
    if(!fPrimaryGeneratorActionBasic->SetBeamTheta(fBeamThetaCmd->GetNewDoubleValue(newValue))){
      G4Exception(0, 0, FatalException, 0);
    }
  }
}

