
#include "RunActionBasic.hh"
#include "ActionBasicMessenger.hh"

#include "G4Run.hh"

#include "TSimulationDataBeam.hh"
#include "TSimulationDataScintillator.hh"
#include "TSimulationParameterGlobal.hh"
#include "TAnalysisInterface.hh"

#include "TFile.h"
#include "TTree.h"

#include <fstream>
#include <iomanip>
#include <sstream>

RunActionBasic::RunActionBasic(TAnalysisInterface* analysisInterface)
  : fAnalysisInterface(analysisInterface), fFileOut(0) 
{
  fActionBasicMessenger = ActionBasicMessenger::GetInstance();
}

RunActionBasic::~RunActionBasic()
{
  delete fAnalysisInterface;
}

void RunActionBasic::BeginOfRunAction(const G4Run* aRun)
{
  G4int id = aRun->GetRunID();
  G4cout << "### Run " << id << " start" << G4endl;

  // accumulate data by ROOT
  std::ostringstream oss;
  oss << "root/" << fActionBasicMessenger->GetRunName() << std::setw(4) << std::setfill('0') << id << ".root";

  // over write warning
  std::ifstream ifileForCheck(oss.str().c_str());
  if(ifileForCheck){
    G4cout << oss.str() << " already exists. over write[y], exit[n]>" << G4endl;    
    std::string yn;
    G4cin >> yn;
    if(!(yn == "y" || yn == "yes")) std::exit(1);
  }
  fFileOut = TFile::Open(oss.str().c_str(), "recreate");
  if(!fFileOut || fFileOut->IsZombie()) G4Exception(0, 0, FatalException, 0);

  fAnalysisInterface->Initialize();

  if(fActionBasicMessenger->GetUseTree()){
    // create event data branch. tree will be automatically added to current directory
    TTree* tree = new TTree(fActionBasicMessenger->GetTreeName(),
			    fActionBasicMessenger->GetTreeTitle());
    fAnalysisInterface->CreateBranch(tree);

    // save tree name
    if(!gSimulationParameterGlobal) gSimulationParameterGlobal = new TSimulationParameterGlobal;
    gSimulationParameterGlobal->fTreeName = fActionBasicMessenger->GetTreeName();
  }

  fAnalysisInterface->AddParameters(fFileOut);
  fAnalysisInterface->ReadParameters(fFileOut);
  fAnalysisInterface->PrintParameters();
  fAnalysisInterface->CreateHist();
}

void RunActionBasic::EndOfRunAction(const G4Run* /*aRun*/)
{
  fFileOut->Write();

  if(fActionBasicMessenger->GetUseTree()){
    TTree* tree;
    gDirectory->GetObject(fActionBasicMessenger->GetTreeName(), tree);
    tree->ResetBranchAddresses();
  }

  G4cout << G4endl;
}
