#ifndef TANALYSISNEUTRONDETECTOR_HH
#define TANALYSISNEUTRONDETECTOR_HH

#include "TObject.h"
#include "TVector3.h"
#include "TString.h"

#include <vector>
#include <string>

#include "TAnalysisInterface.hh"
#include "TSimulationDataScintillator.hh"

class TFile;
class TTree;
class TH1;
class TSimulationParameterGlobal;
class TSimulationParameterNeutronDetector;

enum EModuleType {kNEUT, kVETO, kEXNEUT};

class TAnalysisNeutronDetector : public TAnalysisInterface
{
public:
  TAnalysisNeutronDetector();
  virtual ~TAnalysisNeutronDetector();
  
  int SetNumberOfNeutrons(Int_t numberOfNeutrons);
  int SetVetoMode(Int_t vetoMode);

  virtual int CreateAndAnalyzeExpData();

  virtual int Initialize();
  virtual int CreateBranch(TTree* tree);
  virtual int AddParameters(TFile* file);
  virtual int ReadParameters(TFile* file);
  virtual int PrintParameters();
  virtual int CreateHist();

  virtual int ClearBuffer();
  virtual int ProcessOneEvent();

protected:
  //calculate simlation data to experiment data
  virtual void SimToExp();

protected:
  Int_t fNumberOfNeutrons;
  Int_t fVetoMode;

  TH1* fHMulti;

  TH1* fHIDX;
  TH1* fHIDY;
  TH1* fHIDZ;

  TH1* fHIDQL;
  TH1* fHIDQR;
  TH1* fHIDTL;
  TH1* fHIDTR;
  TH1* fHIDQAve;
  TH1* fHIDTAve;
  std::vector<TH1*> fHTQ;
  std::vector<TH1*> fHQT;
  std::vector<TH1*> fHXY;

  TH1* fhenergy;
  TH1* fhenergytheta;
  TH1* fhenergy1n;
  TH1* fhq1q2;
  TH1* fhdtq1;
  TH1* fhdtq2;
  TH1* fhdbetaq1;
  TH1* fhdbetaq2;
  TH1* fhivdbetaq1;
  TH1* fhivdbetaq2;
  TH1* fhbetaddbetaq1;
  TH1* fhbetaddbetaq2;
  TH1* fhivbetaddbetaq1;
  TH1* fhivbetaddbetaq2;

  TH1* fhd12;
  TH1* fhtheta12;
  TH1* fhv12;

  // parameters
  TSimulationParameterGlobal* fSimParaGlobal;
  TSimulationParameterNeutronDetector* fSimParaNeut;

  Int_t fTotalNeutNumber;
  Int_t fTotalVetoNumber;
  Int_t fTotalModuleNumber;  

  Double_t fNeutHardwareThreshold;
  Double_t fVetoHardwareThreshold;
  Double_t fNeutSoftwareThreshold;
  Double_t fVetoSoftwareThreshold;
  Double_t fTimeResolution;
  Double_t fNeutAttenuationLength;
  Double_t fVetoAttenuationLength;
  Double_t fPropagationVelocity;

  // temporary container
  std::vector<std::vector<Int_t> > fID;
  std::vector<EModuleType> fMT;
  std::vector<Double_t> fQL, fQR;
  std::vector<Double_t> fTL, fTR;

  ClassDef(TAnalysisNeutronDetector, 1)
};

// neutron detector data structure class
class TAnalysisNeutronDetectorData
{
public:
  TAnalysisNeutronDetectorData(){;}
  ~TAnalysisNeutronDetectorData(){;}

  bool operator==(const TAnalysisNeutronDetectorData& data){
    if(this->id == data.id){
      return true;
    }else{
      return false;
    }
  };

  std::vector<int> id;
  int sid;
  EModuleType mt;
  double qu, qd, tu, td;
  double qave, tave, dt;
  TVector3 pos;

  double time;
  TVector3 beta, gamma, p;
  double betaabs, pabs,  energy, theta, phi;

  ClassDef(TAnalysisNeutronDetectorData, 1)
};

// neutron detector analysis tools
class TAnalysisNeutronDetectorTools
{
public:
  static void threshold(std::vector<TAnalysisNeutronDetectorData>& data_v, double neutSoftwareThreshold, double vetoSoftwareThreshold);
  static void acceptance(std::vector<TAnalysisNeutronDetectorData>& data_v, double theta);
  // mode  0:all veto, 1:first only, <0:others 
  static bool veto(std::vector<TAnalysisNeutronDetectorData>& data_v, int mode);
  static void killveto(std::vector<TAnalysisNeutronDetectorData>& data_v);
  static void hitmintime(std::vector<TAnalysisNeutronDetectorData>& data_v);
  static void hitmintime2(std::vector<TAnalysisNeutronDetectorData>& data_v);
  static void hitmintime12(std::vector<TAnalysisNeutronDetectorData>& data_v);

  ClassDef(TAnalysisNeutronDetectorTools, 1)
};

#endif
