
#include "TAnalysisBulk.hh"
#include "TAnalysisTools.hh"

#include "TFile.h"
#include "TDirectory.h"
#include "TTree.h"
#include "TH1D.h"
#include "TH2D.h"
#include "TRandom.h"
#include "TVector3.h"

#include <iostream>
#include <sstream>
#include <string>
#include <cmath>
#include <vector>

#include "TSimulationDataBeam.hh"
#include "TSimulationParameterGlobal.hh"
#include "TSimulationDataScintillator.hh"

ClassImp(TAnalysisBulk)

TAnalysisBulk::TAnalysisBulk()
  : TAnalysisInterface()
{;}

TAnalysisBulk::~TAnalysisBulk()
{;}

int TAnalysisBulk::CreateAndAnalyzeExpData()
{
  TTree* tree;
  fFileSim->GetObject(fSimParaGlobal->fTreeName, tree);
  if(!tree){
    std::cerr << " cannot find tree" << std::endl;
    return 0;
  }

  TBranch* branchSimulationDataScintillatorArray = 0;
  if(tree->SetBranchAddress("neutronDetector",&gSimulationDataScintillatorArray,&branchSimulationDataScintillatorArray) < 0) return 0;

  if(fNumberOfEntries < 0) fNumberOfEntries = tree->GetEntriesFast();
  for(int i=0; i<fNumberOfEntries; i++) {
    if((i+1)%1000 == 0){
      std::cout << "\r analyzed events:" << (i+1) << std::flush;
    }

    Long64_t entry = tree->LoadTree(i);
    branchSimulationDataScintillatorArray->GetEntry(entry);

    ProcessOneEvent();
  }
  std::cout << std::endl;

  tree->ResetBranchAddresses();

  return 1;
}

int TAnalysisBulk::Initialize()
{
  delete gSimulationDataBeamArray; gSimulationDataBeamArray = 0;
  delete gSimulationDataScintillatorArray; gSimulationDataScintillatorArray = 0;
  gSimulationDataBeamArray = new TSimulationDataBeamArray;
  gSimulationDataScintillatorArray = new TSimulationDataScintillatorArray;

  return 1;
}

int TAnalysisBulk::CreateBranch(TTree* tree)
{
  tree->Branch("beam", gSimulationDataBeamArray);
  tree->Branch("bulk", gSimulationDataScintillatorArray);

  return 1;
}

int TAnalysisBulk::AddParameters(TFile* file)
{
  file->Add(gSimulationParameterGlobal);

  return 1;
}

int TAnalysisBulk::ReadParameters(TFile* file)
{
  std::cout << "reading parameters" << std::endl;

  file->GetObject("parameterGlobal", fSimParaGlobal);
  if(!fSimParaGlobal){
    std::cerr << " parameter does not exist" << std::endl;
    return 0;
  }

  return 1;
}

int TAnalysisBulk::PrintParameters()
{
  std::cout << *fSimParaGlobal << "\n"
    	    << std::flush;

  return 1;
}

int TAnalysisBulk::CreateHist()
{
  fHQ = new TH1D("hq", "q", 300, 0, 150);
  fHT = new TH1D("ht", "t", 300, 0, 2);
  fHTQ = new TH2D("htq", "q", 300, 0, 2, 300, 0, 150);

  return 1;
}

int TAnalysisBulk::ClearBuffer()
{
  gSimulationDataScintillatorArray->clear();

  return 1;
}

int TAnalysisBulk::ProcessOneEvent()
{
  if(gSimulationDataScintillatorArray->empty()){
#ifdef DEBUG
    std::cout << " no events" << std::endl;
#endif
    return 0;
  }

  //  std::cout << "aaa" << std::endl;

  // --------------------- create experiment data ---------------------
  double Q = 0;
  double T = 1000000;
  for(int i=0; i<(int)gSimulationDataScintillatorArray->size(); ++i){
    TSimulationDataScintillator simulationDataScintillator = (*gSimulationDataScintillatorArray)[i];
    //    std::cout << simulationDataScintillator << std::endl;

    Double_t energyDepositEE = TAnalysisTools::MeVtoMeVee(simulationDataScintillator.fPreKineticEnergy,
							  simulationDataScintillator.fPreKineticEnergy-
							  simulationDataScintillator.fEnergyDeposit,
							  simulationDataScintillator.fParticleName.Data(),
							  simulationDataScintillator.fPDGCode);

    Q += energyDepositEE;
    if(T > simulationDataScintillator.fPreTime){
      T =  simulationDataScintillator.fPreTime;
    }
  }

  //  std::cout << "aaa" << std::endl;
  if(Q > 0.1){
//    for(int i=0; i<(int)gSimulationDataScintillatorArray->size(); ++i){
//      if(T<0.5 && Q<25){
//	TSimulationDataScintillator simulationDataScintillator = (*gSimulationDataScintillatorArray)[i];
//	std::cout << T<< " " <<Q << " " << simulationDataScintillator << std::endl;
//      }
//    }
    fHQ->Fill(Q);
    fHT->Fill(T);
    fHTQ->Fill(T,Q);
  }

  return 1;
}
