
#include "DALI2Construction.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"
#include "G4VSolid.hh"
#include "G4SubtractionSolid.hh"
#include "G4RotationMatrix.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

#include "ScintillatorSD.hh"
#include "G4SDManager.hh"
#include "G4VSensitiveDetector.hh"

DALI2Construction::DALI2Construction()
  : fLogicDALI2(0), fLogicWorld(0), fScintillatorSD(0)
{
  fDALI2Material = G4NistManager::Instance()->FindOrBuildMaterial("G4_AIR");
  fDALI2Material_Al = G4NistManager::Instance()->FindOrBuildMaterial("G4_Al");
  fDALI2Material_MgO = G4NistManager::Instance()->FindOrBuildMaterial("G4_MAGNESIUM_OXIDE");
  fDALI2Material_NaI = G4NistManager::Instance()->FindOrBuildMaterial("G4_SODIUM_IODIDE");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");

  G4String baseDir = G4String(getenv("SMCONSTRUCTIONDIR")) + "/txt/";
  //  G4String baseDir = "./txt/";
  fDALI2GeometryIn = baseDir + "dali2_geometry_in.txt";
  fDALI2GeometryOut = baseDir + "dali2_geometry_out.txt";
}

DALI2Construction::~DALI2Construction()
{;}

G4VPhysicalVolume* DALI2Construction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Construct DALI2 LV
  fLogicDALI2 = ConstructSub();

  // DALI2 position
  G4ThreeVector dali2Pos(0, 0, 0);

  // World LV
  G4ThreeVector worldSize(2*fDALI2Size.x(),
			  2*fDALI2Size.y(),
			  2*fDALI2Size.z());
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // dali2 PV
  new G4PVPlacement(0,
		    dali2Pos,
		    fLogicDALI2,
		    "DALI2",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicNebula->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

G4LogicalVolume* DALI2Construction::ConstructSub()
{
  G4cout<<"Creating the DALI2"<< G4endl;

  // dali2 mother volume
  G4ThreeVector dali2Size(3*m, 3*m, 3*m);
  fDALI2Size = dali2Size;

  G4Box* solidDALI2 = new G4Box("DALI2",
				dali2Size.x()/2,
				dali2Size.y()/2,
				dali2Size.z()/2);
  G4LogicalVolume* logicDALI2 = new G4LogicalVolume(solidDALI2,fDALI2Material,"DALI2");


  G4int nofcrystals = 186;
 
  G4double fPosX[186];
  G4double fPosY[186];
  G4double fPosZ[186];
  G4double fFIX[186];
  G4double fFIY[186];
  G4double fFIZ[186];
 
  for(int i=0;i<186;i++)  {
    fPosX[i] = fPosY[i] = fPosZ[i] = 0.0;
    fFIX[i] = fFIY[i] = fFIZ[i] = -999.0;
  }
 
  // The Saint-Gobain Housing:
  G4VSolid* sDali2AlHouseOutSG = new G4Box("sDali2AlHouseOutSG",17.64*cm/2.0,8.52*cm/2.0,5.014*cm/2.0);
  G4VSolid* sDali2AlHouseInSG  = new G4Box("sDali2AlHouseInSG",17.44*cm/2.0,8.32*cm/2.0,4.814*cm/2.0);
  G4SubtractionSolid* sDali2AlHouseSG = new G4SubtractionSolid("sDali2AlHouseSG",sDali2AlHouseOutSG,sDali2AlHouseInSG);
  G4LogicalVolume* lDali2AlHouseSG    = new G4LogicalVolume(sDali2AlHouseSG,fDALI2Material_Al,"lDali2AlHouseSG",0,0,0);
  // The Scionix Housing:
  G4VSolid* sDali2AlHouseOutSC = new G4Box("sDali2AlHouseOutSC",18.*cm/2.0,8.3*cm/2.0,4.5*cm/2.0);
  G4VSolid* sDali2AlHouseInSC  = new G4Box("sDali2AlHouseInSC",17.8*cm/2.0,8.1*cm/2.0,4.3*cm/2.0);
  G4SubtractionSolid* sDali2AlHouseSC    = new G4SubtractionSolid("sDali2AlHouseSC",sDali2AlHouseOutSC,sDali2AlHouseInSC);
  G4LogicalVolume* lDali2AlHouseSC    = new G4LogicalVolume(sDali2AlHouseSC,fDALI2Material_Al,"lDali2AlHouseSC",0,0,0);

  //------------------------------
  // The Saint-Gobain Coating:
  G4VSolid* sDali2MgOCoatOutSG = new G4Box("sDali2MgOCoatOutSG",16.28*cm/2.0, 8.28*cm/2.0,4.78*cm/2.0);
  G4VSolid* sDali2MgOCoatInSG  = new G4Box("sDali2MgOCoatInSG",16.*cm/2.0, 8.*cm/2.0,4.5*cm/2.0);
  G4SubtractionSolid* sDali2MgOCoatSG    = new G4SubtractionSolid("sDali2MgOCoatSG",sDali2MgOCoatOutSG,sDali2MgOCoatInSG);
  G4LogicalVolume* lDali2MgOCoatSG    = new G4LogicalVolume(sDali2MgOCoatSG,fDALI2Material_MgO,"lDali2MgOCoatSG",0,0,0);
  // The Scionix Housing:
  G4VSolid* sDali2MgOCoatOutSC = new G4Box("sDali2MgOCoatOutSC",16.4*cm/2.0,8.1*cm/2.0,4.3*cm/2.0);
  G4VSolid* sDali2MgOCoatInSC  = new G4Box("sDali2MgOCoatInSC",16.*cm/2.0, 8.*cm/2.0,4.0*cm/2.0);
  G4SubtractionSolid* sDali2MgOCoatSC    = new G4SubtractionSolid("sDali2MgOCoatSC",sDali2MgOCoatOutSC,sDali2MgOCoatInSC);
  G4LogicalVolume* lDali2MgOCoatSC    = new G4LogicalVolume(sDali2MgOCoatSC,fDALI2Material_MgO,"lDali2MgOCoatSC",0,0,0);
  // -----------------------------
  // the Crystals:
  // The Saint-Gobain:
  G4VSolid* sDali2CrystalSG    = new G4Box("sDali2CrystalSG",16.*cm/2.0,8.*cm/2.0,4.5*cm/2.0);
  G4LogicalVolume* lDali2CrystalSG    = new G4LogicalVolume(sDali2CrystalSG,fDALI2Material_NaI,"lDali2CrystalSG",0,0,0);
  //The Scionix:
  G4VSolid* sDali2CrystalSC    = new G4Box("sDali2CrystalSC",16.*cm/2.0, 8.*cm/2.0,4.*cm/2.0);
  G4LogicalVolume* lDali2CrystalSC    = new G4LogicalVolume(sDali2CrystalSC,fDALI2Material_NaI,"lDali2CrystalSC",0,0,0);
  //_____________________________________________
  // Including also the old dali1 crystals...
  G4VSolid* sDali1AlHouseOut = new G4Box("sDali1AlHouseOut",12.6*cm/2.0,6.6*cm/2.0,6.6*cm/2.0);
  G4VSolid* sDali1AlHouseIn  = new G4Box("sDali1AlHouseIn",12.4*cm/2.0,6.4*cm/2.0,6.4*cm/2.0);
  G4SubtractionSolid* sDali1AlHouse    = new G4SubtractionSolid("sDali1AlHouse",sDali1AlHouseOut,sDali1AlHouseIn);
  G4LogicalVolume* lDali1AlHouse    = new G4LogicalVolume(sDali1AlHouse,fDALI2Material_Al,"lDali1AlHouse",0,0,0);
 
  G4VSolid* sDali1Crystal    = new G4Box("sDali1Crystal",12.*cm/2.0, 6.*cm/2.0,6.*cm/2.0);
  G4LogicalVolume* lDali1Crystal    = new G4LogicalVolume(sDali1Crystal,fDALI2Material_NaI,"lDali1Crystal",0,0,0);
  //_____________________________________________
 
  // Setting the vis attributes:
  //	  G4VisAttributes* visAttRed   = new G4VisAttributes(G4Colour(1.0,0.0,0.0));
  G4VisAttributes* visAttGreen = new G4VisAttributes(G4Colour(0.0,1.0,0.0));
  //	  G4VisAttributes* visAttBlue  = new G4VisAttributes(G4Colour(0.0,0.0,1.0));
  G4VisAttributes* visAttHouse = new G4VisAttributes(G4Colour(1.0,1.0,1.0));
  //	  G4VisAttributes* visAttPMT   = new G4VisAttributes(G4Colour(0.0,0.0,0.0));
 
  lDali2MgOCoatSG->SetVisAttributes(visAttGreen);lDali2MgOCoatSC->SetVisAttributes(visAttGreen);
  lDali2AlHouseSG->SetVisAttributes(visAttHouse);lDali2AlHouseSC->SetVisAttributes(visAttHouse);
 
  float x,y,z,psi,theta,phi,rotSign;
  x = y = z = psi = theta = phi = rotSign = 0.;
  int id;
  int detType = 0;
  G4RotationMatrix Rot3D;
 
  FILE* fFileIn  = fopen(fDALI2GeometryIn,"r");
  FILE* fFileOut = fopen(fDALI2GeometryOut,"w");
  G4cout << "reading DALI2 geometry" << G4endl;
  G4cout << "  input filename: " << fDALI2GeometryIn << G4endl;
  G4cout << "  output filename: " << fDALI2GeometryOut << G4endl;
  if(fFileIn == 0 || fFileOut == 0){
    G4cerr << "cannot open dali2 geometry file" << G4endl;
    G4Exception(0, 0, FatalException, 0);
  }
  for(int i=0;!feof(fFileIn)&&i<186;i++)  {
    fscanf(fFileIn,"%f %f %f %f %f %f %f %i",&x,&y,&z,&psi,&theta,&phi,&rotSign,&detType);
#ifdef DEBUG
    G4cout<<"Read detector "<<i<<": "<<x<<" "<<y<<" "<<z<<" "<<psi<<" "<<theta<<" "<<phi<<" "<<rotSign<<" "<<detType<< G4endl;
#endif
 
    //The first digit is for the array type, digits 2-4 are used for the detector number
    //The last two digits are for the different physical materials of one detector
    id =  100000 +(i+1)*100;
 
    int dali2Det = (id-100000)/100-1;
    G4ThreeVector Pos(x*cm,y*cm,z*cm); //Position of the center
 
    Rot3D.set(0, 0, 0);
    Rot3D.rotateX(psi*degree);
    Rot3D.rotateY(theta*degree);
    Rot3D.rotateZ(phi*degree);
 
    G4ThreeVector AlHousePosition;
    G4ThreeVector  PMTPosition;
    char   fTempname[200];
 
    //The detector type (SG==0,SC==1):
    if(detType==0)  {
      AlHousePosition = Pos + Rot3D(G4ThreeVector(rotSign*0.535*cm,0.,0)); //cout<<"rotSign: "<<rotSign<<endl;
      sprintf(fTempname,"pDali2AlHouse_%i",dali2Det);
      new G4PVPlacement(G4Transform3D(Rot3D,AlHousePosition),lDali2AlHouseSG,fTempname,logicDALI2,false,id+2);
 
      sprintf(fTempname,"pDali2MgOCoat_%i",dali2Det);
      new G4PVPlacement(G4Transform3D(Rot3D,Pos),lDali2MgOCoatSG,fTempname,logicDALI2,false,id+1);
 
      sprintf(fTempname,"pDali2Crystal_%i",dali2Det);
      new G4PVPlacement(G4Transform3D(Rot3D,Pos),lDali2CrystalSG,fTempname,logicDALI2,false,id);
    }
    else if(detType==1)  {
      AlHousePosition = Pos + Rot3D(G4ThreeVector(rotSign*0.7*cm,0.,0));
      sprintf(fTempname,"pDali2AlHouse_%i",dali2Det);
      new G4PVPlacement(G4Transform3D(Rot3D,AlHousePosition),
 			lDali2AlHouseSC,fTempname,logicDALI2,false,id+2);
      sprintf(fTempname,"pDali2MgOCoat_%i",dali2Det);
      new G4PVPlacement(G4Transform3D(Rot3D,Pos),
 			lDali2MgOCoatSC,fTempname,logicDALI2,false,id+1);
      sprintf(fTempname,"pDali2Crystal_%i",dali2Det);
      new G4PVPlacement(G4Transform3D(Rot3D,Pos),
 			lDali2CrystalSC,fTempname,logicDALI2,false,id);
    }
    else if(detType==2)  {
      sprintf(fTempname,"pDali2AlHouse_%i",dali2Det);
      new G4PVPlacement(G4Transform3D(Rot3D,Pos),
 			lDali1AlHouse,fTempname,logicDALI2,false,id+2);
      sprintf(fTempname,"pDali2Crystal_%i",dali2Det);
      new G4PVPlacement(G4Transform3D(Rot3D,Pos),
 			lDali1Crystal,fTempname,logicDALI2,false,id);
    }
    else{G4cerr<<"You chose the wrong detector type"<< G4endl; return 0;}
 
    // Assigning the positions:
    fPosX[dali2Det] = Pos.getX();
    fPosY[dali2Det] = Pos.getY();
    fPosZ[dali2Det] = Pos.getZ();
    //-----------------------------------------
#ifdef DEBUG
    G4cout << id << " " << fPosX[dali2Det]/cm
 	   << " " << fPosY[dali2Det]/cm
 	   << " " << fPosZ[dali2Det]/cm << " " << G4endl;
#endif
 
    float distance = sqrt(fPosX[dali2Det]/mm*fPosX[dali2Det]/mm  +
 			  fPosY[dali2Det]/mm*fPosY[dali2Det]/mm	 +
 			  fPosZ[dali2Det]/mm*fPosZ[dali2Det]/mm);
 
    float thetaPlaced = acos(fPosZ[dali2Det]/mm/distance);
    float phiPlaced   = acos(fPosX[dali2Det]/mm/distance/sin(thetaPlaced));
 
    if(fPosY[dali2Det]/mm < 0.0) phiPlaced = 2.0*3.14159-phiPlaced;
    thetaPlaced = thetaPlaced/3.14159*180.0;
    phiPlaced   = phiPlaced/3.14159*180.0;
 
    if(abs(fPosY[dali2Det]/mm)<1.0 && fPosX[dali2Det]/mm>=0.0) phiPlaced = 0.0;
    if(abs(fPosY[dali2Det]/mm)<1.0 && fPosX[dali2Det]/mm<0.0)  phiPlaced = 180.0;
 
    fprintf(fFileOut,"%i %i %f %f %f \n", dali2Det, detType, thetaPlaced, phiPlaced, distance);
  }
  fclose(fFileIn);
  fclose(fFileOut);
 
  //
  // Sensitive detectors
  //
  if(!fScintillatorSD){
    G4SDManager* SDman = G4SDManager::GetSDMpointer();
    fScintillatorSD = new ScintillatorSD("/ScintillatorDALI2");
    SDman->AddNewDetector(fScintillatorSD);
  }
  lDali2CrystalSG->SetSensitiveDetector(fScintillatorSD);
  lDali2CrystalSC->SetSensitiveDetector(fScintillatorSD);
  lDali1Crystal->SetSensitiveDetector(fScintillatorSD);

  return logicDALI2;
}
