
#include "NeutronWindowConstruction.hh"

#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

#include "G4UnitsTable.hh"
#include "G4ios.hh"
#include "G4ThreeVector.hh"

#include "G4NistManager.hh"

NeutronWindowConstruction::NeutronWindowConstruction()
  : fLogicNeutronWindow(0), fLogicWorld(0)
{
  fNeutronWindowMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_STAINLESS-STEEL");
  fWorldMaterial = G4NistManager::Instance()->FindOrBuildMaterial("G4_Galactic");
}

NeutronWindowConstruction::~NeutronWindowConstruction()
{;}

G4VPhysicalVolume* NeutronWindowConstruction::Construct()
{
  // Cleanup old geometry
  G4GeometryManager::GetInstance()->OpenGeometry();
  G4PhysicalVolumeStore::GetInstance()->Clean();
  G4LogicalVolumeStore::GetInstance()->Clean();
  G4SolidStore::GetInstance()->Clean();

  // Construct NeutronWindow LV
  fLogicNeutronWindow = ConstructSub();

  // neutron window position
  G4ThreeVector neutronWindowPos(0, 0, 6.8*m);

  // World LV
  G4ThreeVector worldSize(2*fNeutronWindowSize.x(),
			  2*fNeutronWindowSize.y(),
			  2*(fNeutronWindowSize.z()+1.2*neutronWindowPos.z()));
  G4Box* solidWorld = new G4Box("World",
				worldSize.x()/2,
				worldSize.y()/2,
				worldSize.z()/2);
  fLogicWorld = new G4LogicalVolume(solidWorld,fWorldMaterial,"World");
  
  // neutron window PV
  new G4PVPlacement(0,
		    neutronWindowPos,
		    fLogicNeutronWindow,
		    "NeutronWindow",
		    fLogicWorld,
		    false,
		    0);

  // World PV
  G4ThreeVector worldPos(0, 0, 0);  
  G4VPhysicalVolume* world = new G4PVPlacement(0,
					       worldPos,
					       fLogicWorld,
					       "World",
					       0,
					       false,
					       0);

  //  fLogicNebula->SetVisAttributes(G4VisAttributes::Invisible);
  //  fLogicWorld->SetVisAttributes(G4VisAttributes::Invisible);

  G4cout <<* (G4Material::GetMaterialTable()) << G4endl;

  return world;
}

G4LogicalVolume* NeutronWindowConstruction::ConstructSub()
{
  std::cout << "Createing Neutron Window" << std::endl;

  // neutron window LV
  G4ThreeVector neutronWindowSize(2*m, 1*m, 3/cos(30*deg)*mm);
  fNeutronWindowSize = neutronWindowSize;

  G4Box* solidNeutronWindow = new G4Box("NeutronWindow",
					neutronWindowSize.x()/2,
					neutronWindowSize.y()/2,
					neutronWindowSize.z()/2);
  G4LogicalVolume* logicNeutronWindow = new G4LogicalVolume(solidNeutronWindow,fNeutronWindowMaterial,"NeutronWindow");

  logicNeutronWindow->SetVisAttributes(new G4VisAttributes(G4Colour::Black()));

  return logicNeutronWindow;
}
