
#include "ScintillatorSD.hh"

#include "G4HCofThisEvent.hh"
#include "G4SDManager.hh"
#include "G4VPhysicalVolume.hh"
#include "G4Step.hh"
#include "G4Track.hh"
#include "G4VTouchable.hh"
#include "G4TouchableHistory.hh"
#include "G4VProcess.hh"
#include "G4ParticleDefinition.hh"
#include "G4PrimaryParticle.hh"

#include "TSimulationDataScintillator.hh"

#include "TFile.h"
#include "TTree.h"
#include "TBranch.h"

ScintillatorSD::ScintillatorSD(G4String name) : G4VSensitiveDetector(name)
{;}

ScintillatorSD::~ScintillatorSD()
{;}

void ScintillatorSD::Initialize(G4HCofThisEvent* /*HCTE*/)
{;}

G4bool ScintillatorSD::ProcessHits(G4Step* aStep, G4TouchableHistory* /*ROhist*/)
{
  G4StepPoint* preStepPoint = aStep->GetPreStepPoint();
  G4StepPoint* postStepPoint = aStep->GetPostStepPoint();
  const G4DynamicParticle* dynamicParticle = aStep->GetTrack()->GetDynamicParticle();
  const G4ParticleDefinition* particleDefinition = aStep->GetTrack()->GetParticleDefinition();
  const G4VProcess* postProcess = postStepPoint->GetProcessDefinedStep();

  // charged particleのenergy lossした時間としてはpresteppointがよい。
  // しかし、反応自体はpoststeppointの方がよい...
  Double_t preTime = preStepPoint->GetGlobalTime();
  Double_t postTime = postStepPoint->GetGlobalTime();
  Double_t energyDeposit = aStep->GetTotalEnergyDeposit();
  Double_t preKineticEnergy = preStepPoint->GetKineticEnergy();
  Double_t postKineticEnergy = postStepPoint->GetKineticEnergy();
  TVector3 prePosition(preStepPoint->GetPosition().x(),
  		       preStepPoint->GetPosition().y(),
  		       preStepPoint->GetPosition().z());
  TVector3 postPosition(postStepPoint->GetPosition().x(),
			postStepPoint->GetPosition().y(),
			postStepPoint->GetPosition().z());
  TVector3 preMomentum(preStepPoint->GetMomentum().x(),
		       preStepPoint->GetMomentum().y(),
		       preStepPoint->GetMomentum().z());
  TVector3 postMomentum(postStepPoint->GetMomentum().x(),
			postStepPoint->GetMomentum().y(),
			postStepPoint->GetMomentum().z());

  G4int primaryParticleID = 0;  // particle ID needs to simulate more than 1 particle injection, but not yet
  G4int pdgCode = dynamicParticle->GetPDGcode();
  TString particleName(particleDefinition->GetParticleName().data());
  TString processName;
  if(postProcess) processName = postProcess->GetProcessName().data();

  G4TouchableHistory* theTouchable = (G4TouchableHistory*)(preStepPoint->GetTouchable());
  std::vector<Int_t> moduleID;
  for(G4int i=0; i<theTouchable->GetHistoryDepth(); ++i){
    G4int depth = i;//0:self, 1:mother, 2:grandmother, ....
    moduleID.push_back(theTouchable->GetCopyNumber(depth));
  }
  G4VPhysicalVolume* physicalVolume = theTouchable->GetVolume(0); // 0: depth
  const G4String& moduleName = physicalVolume->GetName();

  if((energyDeposit>0.) && (aStep->GetTrack()->GetDefinition()->GetPDGCharge() != 0.)){
  //  {
    TSimulationDataScintillator simulationDataScintillator;
    simulationDataScintillator.fSensitiveDetectorName = SensitiveDetectorName;
    simulationDataScintillator.fModuleName = moduleName;
    simulationDataScintillator.fModuleID = moduleID;
    simulationDataScintillator.fPrimaryParticleID = primaryParticleID;
    simulationDataScintillator.fPDGCode = pdgCode;
    simulationDataScintillator.fParticleName = particleName;
    simulationDataScintillator.fProcessName = processName;
    simulationDataScintillator.fPrePosition = 1./cm*prePosition;
    simulationDataScintillator.fPostPosition = 1./cm*postPosition;
    simulationDataScintillator.fPreMomentum = 1./MeV*preMomentum;
    simulationDataScintillator.fPostMomentum = 1./MeV*postMomentum;
    simulationDataScintillator.fPreKineticEnergy = preKineticEnergy/MeV;
    simulationDataScintillator.fPostKineticEnergy = postKineticEnergy/MeV;
    simulationDataScintillator.fPreTime = preTime/ns;
    simulationDataScintillator.fPostTime = postTime/ns;
    simulationDataScintillator.fEnergyDeposit = energyDeposit/MeV;

    gSimulationDataScintillatorArray->push_back(simulationDataScintillator);

    //    G4cout << gSimulationDataScintillatorArray->size() << " " << simulationDataScintillator << G4endl;
#ifdef DEBUG
    G4cout << gSimulationDataScintillatorArray->size() << " " << simulationDataScintillator << G4endl;
#endif
  }
  
  return true;
}

void ScintillatorSD::EndOfEvent(G4HCofThisEvent* /*HCTE*/)
{;}

